/*
 *	@name		srm.h
 *	@author		Ronny Tschueter, TU Dresden
 *	@license	BSD
 */

#ifndef SRM_CLIENT_H_
#define SRM_CLIENT_H_


#include "srm_types.h"
#include "apiHeaderAll.h"

/*
 *	Define SRM_DEBUG for testing/debugging purposes.
 */
#define GRIDFTP_DEBUG

/*
 *	The location of the user proxy certificate within his home directory.
 */
static const char *proxy_file = "/user-proxy";


/******************************
 *	global definitions
 ******************************/

/*
 * Success message
 */
#define OK                          "OK.\n"
/*
 * String to introduce error messages
 */
#define ERROR_INTRO                 "Error: "
/*
 * Connection setup failure
 */
#define INVALID_USER_PROXY_CERT     "\nConnection could not be established.\nPossible reasons: host unreachable, invalid certificate infrastructure\n"
/*
 * ruleExecInfo parameter is NULL
 */
#define NULL_REI_PARAMETER          "Error while testing ruleExecInfo parameter."
/*
 * Failure due to NULL parameters
 */
#define NULL_PARAMETER              "Some input/output parameters are not specified."
/*
 * Parameter type specification error
 */
#define NO_PARAM_TYPE_SPEC          "Some parameters does not contain type specification."
/*
 * Invalid paramter type
 */
#define INVALID_PARAM_TYPE          "Parameters of invalid type."
/*
 * Invalid specification of data transfer protocols
 */
#define INVALID_PROTO_SPEC          "Corrupted specification of transfer protocols."
/*
 * Invalid specification of storage URL
 */
#define INVALID_SURL_SPEC           "Insufficient numbers of storage URLs specified."
/*
 * Out-of-memory exceptions
 */
#define OUT_OF_MEM                  "Out-of-memory exception."
/*
 * Error while converting time parameter. It exceeds valid range.
 */
#define TIME_PARAM_INVALID_RANGE    "Time parameter exceeds valid range."
/*
 * Error while converting size parameter. It exceeds valid range.
 */
#define SIZE_PARAM_INVALID_RANGE    "Size parameter exceeds valid range."
/*
 * Call user to check iRule command
 */
#define CHK_CMD                     "Please check your iRule command."


/*
 * Out of memory exception
 */
#define OUT_OF_MEMORY_EXCEPTION     -99

/*
 * Supported data types of SRM_LONG64
 */
#define SRM_LONG64_IS_LONG          1
#define SRM_LONG64_IS_LONG_LONG     2




/******************************
 *	main srm functions
 ******************************/

/*
 * 	Tests the reachability of a network host.
 *
 *	@param destURL      URL of SRM manager
 *	@return             0 on success, -1 on failure
 */
int srm_ping_func( char *destURL );

/*
 * 	Lists all files of a directory.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_ls_input data structure
 *	@return             number of listed files on success, -1 on failure
 */
int srm_ls_func( char *endpoint, struct srm_ls_input *input );

/*
 * 	Removes specified files.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_rm_input data structure
 *	@return				number of deleted files on success, -1 on failure
 */
int srm_rm_func( char *endpoint, struct srm_rm_input *input );

/*
 * 	Creates a new directory.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_mkdir_input data structure
 *	@return				0 on success, -1 on failure
 */
int srm_mkdir_func( char *endpoint, struct srm_mkdir_input *input );

/*
 * 	Deletes an empty directory.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_rmdir_input data structure
 *	@return				1 on success, -1 on failure
 */
int srm_rmdir_func( char *endpoint, struct srm_rmdir_input *input );

/*
 * 	Gets user and group permissions for specified file.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_getpermission_input data structure
 *	@return				number of file permissions on success, -1 in failure
 */
int srm_getpermission_func( char *endpoint, struct srm_getpermission_input *input );

/*
 * 	Sets user and group permissions for specified file.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_setpermission_input data structure
 *	@return				0 on success, -1 on failure
 */
int srm_setpermission_func( char* endpoint, struct srm_setpermission_input *input );

/*
 * 	Tranfers data to online media.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_bringonline_input data structure
 *	@return				number of files on success, -1 on failure
 */
int srm_bringonline_func( char* endpoint, struct srm_bringonline_input *input );

/*
 * 	Returns the spacetoken with the biggest sufficient space.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_getbestspacetokens_input data structure
 *	@return				0 on success, -1 on failure
 */
int srm_getbestspacetoken_func( char* endpoint, struct srm_getbestspacetokens_input *input );

/*
 * 	Returns all space tokens of the specified token descriptor
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_getspacetokens_input data structure
 *	@return				nuber of spacetokens on success, -1 on failure
 */
int srm_getspacetokens_func( char* endpoint, struct srm_getspacetokens_input *input );

/*
 * 	Returns all space meta data for the requested space tokens
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_getspacemd_input data structure
 *	@return				number of meta data information on success, -1 on failure
 */
int srm_getspacemd_func( char* endpoint, struct srm_getspacemd_input *input );

/*
 * 	Prepares 'put' data transfer operation.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_preparetoput_input data structure
 *	@return				number of files on success, -1 on failure
 */
int srm_preparetoput_func( char* endpoint, struct srm_preparetoput_input *input );

/*
 * 	Checks whether a prior 'put' data transfer operation has finished.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_putdone_input data structure
 *	@return				number of files on success, -1 on failure
 */
int srm_put_done_func( char* endpoint, struct srm_putdone_input *input );

/*
 * 	Prepares 'get' data transfer operation.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_preparetoget_input data structure
 *	@return				number of files on success, -1 on failure
 */
int srm_preparetoget_func( char* endpoint, struct srm_preparetoget_input *input );

/*
 * 	Aborts an selective file request.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_abort_files_input data structure
 *	@return				number of files on success, -1 on failure
 */
int srm_abortfiles_func( char* endpoint, struct srm_abort_files_input *input );

/*
 * 	This function is used to release pins on previously requested “copies” of the SURL.
 *
 *	@param endpoint     URL of SRM manager
 *	@param input        srm_releasefiles_input data structure
 *	@return				number of files on success, -1 on failure
 */
int srm_releasefiles_func( char* endpoint, struct srm_releasefiles_input *input );


/***************************
 *	helper functions
 ***************************/

/*
 *	Initializes context of SRM function calls.
 *
 *	@param context	        srm_context data structure
 *	@param endpoint	        URL of SRM manager
 *	@return			        0 on success, -1 on failure
 */
int initialize_srm_context( struct srm_context *context,char *srm_endpoint );

/*
 *	Sets the user proxy certificate for use within microservices.
 *
 *	@param rsComm	        iRODS server communication handle
 *	@return			        0 on success, 1 on failure
 */
int setUserProxy( rsComm_t *rsComm );

/*
 *  Splits a string into separated tokens.
 *
 *	@param srcString        string that should be sperareted into tokens
 *  @param dstArray         array with all tokens
 *  @param splitter         token delimiter
 *	@return                 number of tokens in dstArray
 */
int splitString( char* srcString, char*** dstArray, char* splitter );

/*
 *  Determines type of SRM_LONG64.
 *  
 *  @return                 SRM_LONG64_IS_LONG if SRM_LONG64 has type long, or
 *                          SRM_LONG64_IS_LONG_LONG if SRM_LONG64 is of type long long
 */
int typeof_SRM_LONG64();

/*
 *  Print all information from srmv2_pinfilestatus data structure to given string.
 *
 *	@param index            integer, can be used to number output
 *  @param str              output string
 *	@param status           srmv2_pinfilestatus data structure containing all information
 */
void printSrmPinFileStatus( int index, char** str, struct srmv2_pinfilestatus status );

/*
 *  Print all information from srmv2_filestatus data structure to given string.
 *
 *	@param index            integer, can be used to number output
 *  @param str              output string
 *	@param status           srmv2_filestatus data structure containing all information
 */
void printSrmFileStatus( int index, char** str, struct srmv2_filestatus status );

/*
 *	Prints an informational message to stdout/rodsLog, if
 *	SRM_DEBUG is defined.
 *
 *	@param msg		message to print
 */
void printInfo( char *msg, ... );

#endif /* SRM_CLIENT_H_ */
