/*
 *  @name       srmMS.h
 *  @author     Ronny Tschueter, TU Dresden
 *  @license    BSD
 */

#ifndef SRM_CLIENT_H_
#define SRM_CLIENT_H_

/*
 * 	iRODS Microservice function that aborts an selective file request.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param requestToken msParam_t containing token of prior 'put' operation
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmAbortFiles( msParam_t *endpoint, msParam_t *surls, msParam_t *requestToken, msParam_t *result, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that aborts an existing request.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param requestToken msParam_t containing token of prior 'put' operation
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmAbortRequest( msParam_t *endpoint, msParam_t *requestToken, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that transfers data online.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param time         msParam_t containing desired pin time
 *  @param desc         msParam_t containing space descriptor
 *  @param protocols    msParam_t containing list of data transfer protocols
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmBringOnline( msParam_t *endpoint, msParam_t *surls, msParam_t *time, msParam_t *desc, msParam_t *protocols, msParam_t *result, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that returns the spacetoken with the biggest sufficient space.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param desc         msParam_t containing space descriptor
 *  @param size         msParam_t containing requested storage size
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmGetBestSpaceToken( msParam_t *endpoint, msParam_t *desc, msParam_t *size, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that gets owner and group permissions for specified file.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmGetPermission( msParam_t *endpoint, msParam_t *surls, msParam_t *result, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that returns all space meta data for the requested space tokens.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param spacetokens  msParam_t containing a list of space tokens
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmGetSpaceMD( msParam_t *endpoint, msParam_t *spacetokens, msParam_t *result, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that returns all space tokens of the specified token descriptor.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param desc         msParam_t containing space descriptor
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmGetSpaceTokens( msParam_t *endpoint, msParam_t *desc, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that lists all files in a directory.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param subdirlevel  msParam_t containing number of listed directory levels
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmLs( msParam_t *endpoint, msParam_t *surls, msParam_t *subdirlevel, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that creates a new directory.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmMkDir( msParam_t *endpoint, msParam_t *surl, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that tests the reachability of a network host.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmPing(msParam_t *endpoint, msParam_t *result, ruleExecInfo_t *rei);

/*
 * 	iRODS Microservice function that prepares 'get' data transfer operation.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param desc         msParam_t containing space descriptor
 *  @param time         msParam_t containing desired pin time
 *  @param protocols    msParam_t containing list of data transfer protocols
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmPrepareToGet( msParam_t *endpoint, msParam_t *surls, msParam_t *desc, msParam_t *time, msParam_t *protocols, msParam_t *result, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that prepares 'put' data transfer operation.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param filesizes    msParam_t lists size of each file to transfer
 *  @param surls        msParam_t containing list of host URLs
 *  @param desc         msParam_t containing space descriptor
 *  @param time         msParam_t containing desired pin time
 *  @param protocols    msParam_t containing list of data transfer protocols
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmPrepareToPut( msParam_t *endpoint, msParam_t *filesizes, msParam_t *surls, msParam_t *desc, msParam_t *time, msParam_t *protocols, msParam_t *result, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that checks whether a prior 'put' data transfer operation has finished.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param requestToken msParam_t containing token of prior 'put' operation
 *  @param surls        msParam_t containing list of host URLs
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmPutDone( msParam_t *endpoint, msParam_t *requestToken, msParam_t *surls, msParam_t *result, ruleExecInfo_t *rei );

/*
 * 	iRODS Microservice function that is used to release pins on previously requested “copies” of the SURL.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param requestToken msParam_t containing token of prior 'put' operation
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmReleaseFiles( msParam_t *endpoint, msParam_t *surls, msParam_t *requestToken, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that deletes files.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmRm( msParam_t *endpoint, msParam_t *surl, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that deletes an directory.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surls        msParam_t containing list of host URLs
 *  @param recursive    msParam_t specifing whether sub-directories should be deleted (1) or not (0)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmRmDir( msParam_t *endpoint, msParam_t *surls, msParam_t *recursive, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that adds new Group permissions
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param groupName    msParam_t containing group name
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmAddGroupPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *groupName, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that replaces existing Group permissions with specified ones
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param groupName    msParam_t containing group name
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmChangeGroupPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *groupName, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that removes specified Group Permissions from already existing ones.
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param groupName    msParam_t containing group name
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmRemoveGroupPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *groupName, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that adds new Other permissions
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmAddOtherPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that replaces existing Other permissions with specified ones
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmChangeOtherPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that removes specified Other permissions from already existing ones
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmRemoveOtherPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that adds new Owner permissions
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmAddOwnerPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that replaces existing Owner permissions with specified ones
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmChangeOwnerPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that removes specified Owner permissions from already existing ones
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param permission   msParam_t containing permission (e.g., r, rw, wx, ...)
 *  @param result       msParam_t for result output
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmRemoveOwnerPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that adds new User permissions
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmAddUserPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *userName, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that replaces existing User permissions with specified ones
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmChangeUserPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *userName, msParam_t *permission, msParam_t *result, ruleExecInfo_t *rei );

/*
 *  iRODS Microservice function that removes specified User permissions from already existing ones
 *
 *  @param endpoint     msParam_t containing SRM manager URL
 *  @param surl         msParam_t containing host URL
 *  @param rei          iRODS ruleExecutionInfo structure
 *  @return             1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 */
int msiSrmRemoveUserPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *userName, msParam_t *permissionString, msParam_t *result, ruleExecInfo_t *rei );

#endif /* SRM_CLIENT_H_ */
