#include <limits.h>

#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


/*
 * Set destination of debug output
 */
#define DEBUG_OUT       stdout

#define SPLITTER        ","
#define CORE_ERROR      "\nPlease check specified storage URL.\n"
#define TOKEN_ERROR     "No information available"
#define TOKEN_HEADER    " Request Token: "


char* resultString;

int srm_bringonline_func( char* endpoint, struct srm_bringonline_input *input ) {

    int status;
    int len;
    int i;

	struct srm_context context;
    struct srm_bringonline_output output;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_bring_online() returns the number of files or -1 for error
     */
    status = srm_bring_online( &context, input, &output );
    if ( status < 0 ) {
        /*
         * An error occurred - we will try multiple ways to get a meaningful error message
         */
        if ( output.retstatus != NULL && output.retstatus->explanation != NULL ) {
            /*
             * First: Use status information from web service to generate an error message
             */
            if ( output.retstatus->statusCode == SRM_USCOREFAILURE ) {
                /*
                 * Maybe user specified a wrong SURL
                 */
                len = strlen(output.retstatus->explanation) + strlen(CORE_ERROR);
                resultString = (char* ) calloc( len + 1, sizeof(char) );
                if ( resultString == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( resultString, len, "%s%s", output.retstatus->explanation, CORE_ERROR );
            }
            else {
                len = strlen(output.retstatus->explanation) + strlen(".\n");
                resultString = (char* ) calloc( len + 1, sizeof(char) );
                if ( resultString == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( resultString, len, "%s\n", output.retstatus->explanation );
            }
            return status;
        }
        else if ( context.errbuf != NULL ) {
            /*
             * Second: Use information from the SRM context error buffer
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char* ) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
        }
        else {
            /*
             * Finally: If there is no return status and no message in the error buffer,
             * something went wrong while setting up connection to web service
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
            return status;
        }
    }
    else {
        /*
         * Allocate memory for result string
         */
        resultString = (char *) calloc( 50000, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }

        /*
         * Fill result string
         */
        if (output.token != NULL ) {
            snprintf( resultString, strlen("\n") + strlen(TOKEN_HEADER) + strlen(output.token) + strlen("\n"), "\n%s%s\n", TOKEN_HEADER, output.token );
        }
        else {
            snprintf( resultString, strlen("\n") + strlen(TOKEN_HEADER) + strlen(TOKEN_ERROR) + strlen("\n"), "\n%s%s\n", TOKEN_HEADER, TOKEN_ERROR );
        }

        for ( i = 0; i < status; i++ ) {
            /*
             * Print all information about srmv2_pinfilestatus
             */
            printSrmPinFileStatus( i, &resultString, output.filestatuses[ i ] );
        }
        return status;
    }
}

/**
 * \fn                  msiSrmBringOnline
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surls - List of storage URLs
 * \param[in STR_MS_T]  time - desired pin time
 * \param[in STR_MS_T]  desc - space descriptor
 * \param[in STR_MS_T]  protocols - list of data transfer protocols
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmBringOnline( msParam_t *endpoint, msParam_t *surls, msParam_t *time, msParam_t *desc, msParam_t *protocols, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

	int status;
    int nr_surls;
    int nr_protocols;
#ifdef SRM_DEBUG
    int debug_index;
#endif

    long int time_li;

    char** surlsArray       = NULL;
    char** protocolsArray   = NULL;
    char** end_ptr;
    char*  spaceTokenDesc;

    struct srm_bringonline_input input;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmBringOnline: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL 
         || surls == NULL
         || time == NULL
         || desc == NULL
         || protocols == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmBringOnline: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL 
         || surls->type == NULL 
         || desc->type == NULL
         || time->type == NULL
         || protocols->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmBringOnline: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surls->type, STR_MS_T ) != 0
         || strcmp( desc->type, STR_MS_T ) != 0
         || strcmp( time->type, STR_MS_T ) != 0
         || strcmp( protocols->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmBringOnline: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Check time parameter
     */
    errno = 0;
    end_ptr = NULL;
    time_li = strtol( (char *) time->inOutStruct, end_ptr, 10 );
    if ( (errno == ERANGE && (time_li == LONG_MAX || time_li == LONG_MIN))
         || (errno != 0 && time_li == 0)
         || time_li < 0
         || time_li > INT_MAX
         || end_ptr != NULL ) {

        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmBringOnline: %s %s", TIME_PARAM_INVALID_RANGE, CHK_CMD );
        return rei->status;
    }

    /*
     * Split SURLs string up into separated items (each item contains one SURL)
     */
    nr_surls = splitString( surls->inOutStruct, &surlsArray, SPLITTER );
    if ( nr_surls <= 0 ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmBringOnline: %s %s", INVALID_SURL_SPEC, CHK_CMD );
		return rei->status;
    }

    /*
     * Split protocols string up into separated items (each item contains one protocol)
     */
    nr_protocols = splitString( protocols->inOutStruct, &protocolsArray, SPLITTER );
    if ( nr_protocols <= 0 ) {
        free( surlsArray );
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmBringOnline: %s %s", INVALID_PROTO_SPEC, CHK_CMD );
        return rei->status;
    }
    // Resize protocolsArray for one additional element (array should finish with an NULL element)
    protocolsArray = realloc( protocolsArray, (nr_protocols + 1) * sizeof(char*) );
    if ( protocolsArray == NULL ) {
        free( surlsArray );
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmBringOnline: %s", OUT_OF_MEM );
        return rei->status;
    }
    // Set last element to NULL
    protocolsArray[ nr_protocols ] = NULL;

    /*
     * Set up srm_bringonline_input data structure
     */
    input.nbfiles           = nr_surls;
    input.surls             = surlsArray;
    input.protocols         = protocolsArray;
    input.desiredpintime    = (int) time_li;
    spaceTokenDesc          = (char* ) desc->inOutStruct;
    if ( strcmp( spaceTokenDesc, "null" ) == 0 ) {
        input.spacetokendesc    = NULL;
    }
    else {
        input.spacetokendesc    = spaceTokenDesc;
    }

#ifdef SRM_DEBUG
    /*
     * Debug output
     */
    debug_index = 0;
    while (1) {
        if ( input.protocols[ debug_index ] == NULL )
        {
            fprintf( DEBUG_OUT, "Protocol [%d]:\tNULL\n", debug_index );
            break;
        }
        else {
            fprintf( DEBUG_OUT, "Protocol [%d]:\t%s\n", debug_index, input.protocols[ debug_index ] );        
            debug_index++;
        }
    }
    fprintf( DEBUG_OUT, "Pin Time:\t\t%d\n", input.desiredpintime );
    fprintf( DEBUG_OUT, "Space Token:\t\t%s\n", input.spacetokendesc );
    fprintf( DEBUG_OUT, "Nr. of Files:\t%d\n", input.nbfiles );
    for ( debug_index = 0; debug_index < input.nbfiles; debug_index++ ) {
        fprintf( DEBUG_OUT, "File [ %d ]:\t%s\n", debug_index, input.surls[ debug_index ] );
    }
#endif

    status = srm_bringonline_func( endpoint->inOutStruct, &input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}
