#include <stdlib.h>
#include <errno.h>

#include "srm.h"
#include "srmMS.h"
#include "globus_common.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define NO_TOKEN_RETURNED   "No token was returned.\n"
#define ERANGE_ERROR        "Value of size parameter exceed valid range.\n"

char* resultString;

int srm_getbestspacetoken_func( char* endpoint, struct srm_getbestspacetokens_input *input ) {

    int len;

    char* token = NULL;

	struct srm_context context;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_getbestspacetoken() returns the spacetoken with the biggest sufficient space
     */
    token = srm_getbestspacetoken( &context, input );
    if ( token == NULL ) {
        if ( context.errbuf != NULL ) {
            // use information from SRM context error buffer
            len = strlen(ERROR_INTRO) + strlen(NO_TOKEN_RETURNED) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, NO_TOKEN_RETURNED, context.errbuf );
        }
        else {
            len = strlen(ERROR_INTRO) + strlen(NO_TOKEN_RETURNED);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, NO_TOKEN_RETURNED );
        }
        return -1;
    }
    else {
        len = strlen(OK) + strlen("\n") + strlen(token) + strlen("\n");
        resultString = (char *) calloc( len + 1, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }
        snprintf( resultString, len, "%s\n%s\n", OK, token );
        return 0;
    }
}

/**
 * \fn                  msiSrmGetBestSpaceToken
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  desc - space descriptor
 * \param[in STR_MS_T]  size - requested storage size
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmGetBestSpaceToken( msParam_t *endpoint, msParam_t *desc, msParam_t *size, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

	int status;

    long long int size_lli;

    char** end_ptr;

    struct srm_getbestspacetokens_input input;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmGetBestSpaceToken: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL 
         || desc == NULL
         || size == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL 
         || desc->type == NULL
         || size->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( desc->type, STR_MS_T ) != 0
         || strcmp( size->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Check size parameter
     */
    errno = 0;
    end_ptr = NULL;
    size_lli = strtoll( (char *) size->inOutStruct, end_ptr, 10 );
    if ( (errno == ERANGE && (size_lli == LLONG_MAX || size_lli == LLONG_MIN))
         || (errno != 0 && size_lli == 0)
         || size_lli < 0
         || size_lli > INT_MAX
         || end_ptr != NULL ) {

        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s %s", SIZE_PARAM_INVALID_RANGE, CHK_CMD );
        return rei->status;
    }

    /*
     * Set up srm_getbestspacetokens_input data structure
     */
    input.spacetokendesc = (char* ) desc->inOutStruct;
    input.neededsize = size_lli;

    status = srm_getbestspacetoken_func( endpoint->inOutStruct, &input );
    switch ( status ) {
        case -1:
            /* FAILURE */
            rei->status = EXEC_CMD_ERROR;
            rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s", resultString );
            fillStrInMsParam( resultParameter, resultString );
            break;
        case OUT_OF_MEMORY_EXCEPTION:
            /* NO MORE MEMORY */
            rei->status = EXEC_CMD_ERROR;
            rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetBestSpaceToken: %s", OUT_OF_MEM );
            break;
        case 0:
            /* SUCCESS */
            rei->status = 1;
            fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}
