#include <stdlib.h>
#include <errno.h>

#include "srm.h"
#include "srmMS.h"
#include "globus_common.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define DEFAULT_ERROR_MSG   "srm_getspacetokens() failed.\n"
#define INSUFF_NR_TOKENS    "Number of returned space tokens <= 0.\n"
#define TOKEN_HEADER        "\nSpace Tokens:\n"

char* resultString;

int srm_getspacetokens_func( char* endpoint, struct srm_getspacetokens_input *input ) {

    int status;
    int len;
    int i;

	struct srm_context context;
    struct srm_getspacetokens_output output;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_getspacetokens() returns all space tokens of the token descriptor or -1 for error
     */
    status = srm_getspacetokens( &context, input, &output );
    if ( status < 0 ) {
        /*
         * Error case
         */
        if ( context.errbuf != NULL ) {
            // use information from SRM context error buffer
            len = strlen(ERROR_INTRO) + strlen(DEFAULT_ERROR_MSG) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, DEFAULT_ERROR_MSG, context.errbuf );
        }
        else {
            len = strlen(ERROR_INTRO) + strlen(DEFAULT_ERROR_MSG);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, DEFAULT_ERROR_MSG );
        }
        return status;
    }
    else {
        /*
         * Check number of returned space tokens
         */
        if ( output.nbtokens <= 0 ) {
            len = strlen(ERROR_INTRO) + strlen(INSUFF_NR_TOKENS) + 1;
            resultString = (char *) calloc( len, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INSUFF_NR_TOKENS );
            return status;
        }

        /*
         * Print all returned space tokens
         */
        len = strlen(TOKEN_HEADER);
        for ( i = 0; i < output.nbtokens; i++ ) {
            len = len + strlen(output.spacetokens[ i ]) + strlen("\n");
        }
        resultString = (char *) calloc( len + 1, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }
        snprintf( resultString, strlen(TOKEN_HEADER), "%s", TOKEN_HEADER );
        for ( i = 0; i < output.nbtokens; i++ ) {
            strncat( resultString, output.spacetokens[ i ], strlen(output.spacetokens[ i ]) );
            strncat( resultString, "\n", strlen("\n") );
        }
        return status;
    }
}

/**
 * \fn                  msiSrmGetSpaceTokens
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  desc - space descriptor
 * \param[out STR_MS_T] resultParameter -: output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmGetSpaceTokens( msParam_t *endpoint, msParam_t *desc, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

	int status;

    struct srm_getspacetokens_input input;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmGetSpaceToken: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL 
         || desc == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceToken: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL 
         || desc->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceToken: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( desc->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceToken: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Fill input data structure
     */
    input.spacetokendesc = (char* ) desc->inOutStruct;

    status = srm_getspacetokens_func( endpoint->inOutStruct, &input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceToken: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmGetSpaceToken: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}
