#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define SPLITTER                ","
#define DIGITS                  22
#define DIR_HEADER              "\nDirectory: "
#define FILES_HEADER            "\nFiles:\n"
#define SURL_HEADER             "SURL:           "
#define STATUS_HEADER           "Status:         "
#define EXPLANATION_HEADER      "Explanation:    "
#define TYPE_HEADER             "Type:           "
#define SIZE_HEADER             "Size:           "
#define PERMISSIONS_HEADER      "Permissions:    "
#define LOCALITY_HEADER         "Locality:       "
#define NR_SUBPATHS_HEADER      "Nr of Subpaths: "
#define CHECKSUM_HEADER         "Checksum:       "
#define CHECKSUM_TYPE_HEADER    "Checksum Type:  "
#define SPACE_TOKEN_HEADER      "Space Token:    "
#define NO_DIR_ERROR            "\nMaybe the specified directory does not exist.\n"
#define STATUS_ERR_MSG          "No file information available.\n"
#define INVALID_LEVEL_SPEC      "Level of subdirectories (3rd argument) is out of range.\n"


char* listing;
char rwx[] = "rwxrwxrwx";
/*
 * NOTE: Implementation detail of srm-ifce library:
 * Usage of struct64 is very strange in srm-ifce
 * library. The library does not use the default
 * fields to store Read, Write, and Execute permissions
 * for user, group and others. Therefore you have to
 * define your own mask/flags to check permissions.
 */
unsigned short perms[] = {
    0x100, // S_IRUSR
    0x080, // S_IWUSR
    0x040, // S_IXUSR
    0x020, // S_IRGRP
    0x010, // S_IWGRP
    0x008, // S_IXGRP
    0x004, // S_IROTH
    0x002, // S_IWOTH
    0x001  // S_IXOTH
};


/*
 * Create a result message with, which contains srmv2_mdfilestatus information.
 *
 * @param str       result message
 * @param fs        srmv2_mdfilestatus data structure
 * @param level     integer for nice message alignment
 * @return          0 on success, otherwise an error occurred (e.g., OUT_OF_MEMORY_EXCEPTION)
 */
int createResultString( char** str, struct srmv2_mdfilestatus* fs, int level ) {

    int     i;
    int     status;
    char    intToString[DIGITS];
    /*
     * string with all permissions (Read, Write, eXecute) for recent item
     * this string stroed permissions in the order user, group, and others
     */
    char    current_rwx_permissions[10];
    char*   pre;

    /*
     * Use tabulators to indent output for items in subdirectories
     */
    if ( level > 0 ) {
        pre = (char* ) calloc( level + 1, sizeof(char) );
        if ( pre == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }
        for ( i = 0; i < level; i++ ) {
            strncat( pre, "\t", strlen("\t") );
        }
    }
    else {
        pre = (char* ) calloc( 1, sizeof(char) );
        if ( pre == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }
    }

    strncat( *str, "\n", strlen("\n") );

    /*
     * SURL
     */
    strncat( *str, pre, strlen(pre) );
    strncat( *str, SURL_HEADER, strlen(SURL_HEADER) );
    strncat( *str, fs->surl, strlen(fs->surl) );
    strncat( *str, "\n", strlen("\n") );

    if ( fs->status != 0 ) {
        /*
         * Status is not OK, print an explanation which may help to pinpoint error
         *
         * NOTE: Implementation detail of srm-ifce library:
         * explanation string is only set, if status is not 0
         */
        strncat( *str, pre, strlen(pre) );
        strncat( *str, STATUS_HEADER, strlen(STATUS_HEADER) );
        snprintf( intToString, DIGITS, "%d\n", fs->status );
        strncat( *str, intToString, strlen(intToString) );

        strncat( *str, pre, strlen(pre) );
        strncat( *str, EXPLANATION_HEADER, strlen(EXPLANATION_HEADER) );
        if ( fs->explanation != NULL ) {
            strncat( *str, fs->explanation, strlen(fs->explanation) );
            strncat( *str, "\n", strlen("\n") );
        }
        else {
            strncat( *str, "No information\n", strlen("No information\n") );
        }
    }

    /*
     * stat64 DATA STRUCTURE
     */
        // TYPE
    strncat( *str, pre, strlen(pre) );
    strncat( *str, TYPE_HEADER, strlen(TYPE_HEADER) );
    if ( fs->stat.st_mode & S_IFREG ) {
        strncat( *str, "File\n", strlen("File\n") );
    } else if ( fs->stat.st_mode & S_IFDIR ) {
        strncat( *str, "Directory\n", strlen("Directory\n") );
    } else if ( fs->stat.st_mode & S_IFLNK ) {
        strncat( *str, "Link\n", strlen("Link\n") );
    } else {
        strncat( *str, "Unknown type\n", strlen("Unknown type\n") );
    }
        // SIZE
    strncat( *str, pre, strlen(pre) );
    strncat( *str, SIZE_HEADER, strlen(SIZE_HEADER) );
    snprintf( intToString, DIGITS, "%llu", (unsigned long long)fs->stat.st_size );
    strncat( *str, intToString, strlen(intToString) );
    strncat( *str, "\n", strlen("\n") );
        // PERMISSIONS
    strncat( *str, pre, strlen(pre) );
    strncat( *str, PERMISSIONS_HEADER, strlen(PERMISSIONS_HEADER) );
    for ( i = 0; i < 9; i++ ) {
        if ( (fs->stat.st_mode & perms[ i ]) != 0 ) {
            // if this permissions (r,w or x) is granted, copy string representation to result string
            current_rwx_permissions[ i ] = rwx[ i ];
        }
         else {
            // if this permissions (r,w or x) is not granted, result string will contain "-" to signalize a missing permission
            current_rwx_permissions[i] = '-';
        }
    }
    // finalize string, which contains all permissions
    current_rwx_permissions[ 9 ] = '\0';
    // concatenate permissions representation to result string
    strncat( *str, current_rwx_permissions, strlen(current_rwx_permissions) );
    strncat( *str, "\n", strlen("\n") );
    /*
     * NOTE: Implementation detail of srm-ifce library:
     * Furthermore srm-ifce library sets stat.st_uid, stat.st_gid, and stat.st_nlink.
     * However, stat.st_uid and stat.st_gid are always set to 2, stat.nlink is constantly set to 1.
     * So these fields don't contain meaningful information and are skipped at this point.
     */

    // TODO: probably Locality implementation is buggy in srm-ifce library
    /*
     * LOCALITY
     */
    strncat( *str, pre, strlen(pre) );
    strncat( *str, LOCALITY_HEADER, strlen(LOCALITY_HEADER) );
    switch ( fs->locality ) {
        case srm_file_locality_UNKNOWN:
            strncat( *str, "Unknown\n", strlen("Unknown\n") );
            break;
        case srm_file_locality_ONLINE:
            strncat( *str, "Online\n", strlen("Online\n") );
            break;
        case srm_file_locality_NEARLINE:
            strncat( *str, "Nearline\n", strlen("Nearline\n") );
            break;
        case srm_file_locality_ONLINE_USCOREAND_USCORENEARLINE:
            strncat( *str, "Online and Nearline\n", strlen("Online and Nearline\n") );
            break;
        case srm_file_locality_LOST:
            strncat( *str, "Lost\n", strlen("Lost\n") );
            break;
        case srm_file_locality_NONE:
            strncat( *str, "None\n", strlen("None\n") );
            break;
        case srm_file_locality_UNAVAILABLE:
            strncat( *str, "Unavailable\n", strlen("Unavailable\n") );
            break;
        default:
            strncat( *str, "Unsupported Type\n", strlen("Unsupported Type\n") );
    }

    /*
     * CHECKSUM AND CHECKSUM TYPE
     *
     * NOTE: Implementation detail of srm-ifce library:
     * Either both (fs->cheksumtype and fs->checksum) are set or none of them is set
     */
    strncat( *str, pre, strlen(pre) );
    strncat( *str, CHECKSUM_TYPE_HEADER, strlen(CHECKSUM_TYPE_HEADER) );
    if ( fs->checksumtype != NULL && fs->checksum != NULL ) {
        strncat( *str, fs->checksumtype, strlen(fs->checksumtype) );
        strncat( *str, "\n", strlen("\n") );
        strncat( *str, pre, strlen(pre) );
        strncat( *str, CHECKSUM_HEADER, strlen(CHECKSUM_HEADER) );
        strncat( *str, fs->checksum, strlen(fs->checksum) );
        strncat( *str, "\n", strlen("\n") );
    }
    else {
        strncat( *str, "No information\n", strlen("No information\n") );
        strncat( *str, pre, strlen(pre) );
        strncat( *str, CHECKSUM_HEADER, strlen(CHECKSUM_HEADER) );
        strncat( *str, "No information\n", strlen("No information\n") );
    }

    /*
     * SPACE TOKENS
     */
    strncat( *str, pre, strlen(pre) );
    strncat( *str, SPACE_TOKEN_HEADER, strlen(SPACE_TOKEN_HEADER) );
    if ( fs->nbspacetokens == 0 ) {
        strncat( *str, "None\n", strlen("None\n") );
    }
    else {
        for ( i = 0; i < fs->nbspacetokens; i++ ) {
            strncat( *str, fs->spacetokens[ i ], strlen(fs->spacetokens[ i ]) );
        }
    }

    /*
     * SUBPATHS
     */
    strncat( *str, pre, strlen(pre) );
    strncat( *str, NR_SUBPATHS_HEADER, strlen(NR_SUBPATHS_HEADER) );
    snprintf( intToString, DIGITS, "%d", fs->nbsubpaths );
    strncat( *str, intToString, strlen(intToString) );
    strncat( *str, "\n", strlen("\n") );
    for ( i = 0; i < fs->nbsubpaths; i++ ) {
        status = createResultString( str, &(fs->subpaths[ i ]), level + 1 );
        if ( status != 0 ) {
            return status;
        }
    }

    free( pre );

    return 0;
}

int srm_ls_func( char *endpoint, struct srm_ls_input *input_ls ) {

	int status;
    int len;
    int i;

	struct srm_context context;
	struct srm_ls_output output_ls;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     *  srm_ls() returns the number of files in the list or -1 for error
     */
    status = srm_ls( &context, input_ls, &output_ls );

	if ( status < 0 ) {
        /*
         * Failure - try multiple ways to get a meaningful error message
         */
        if ( output_ls.retstatus != NULL && output_ls.retstatus->explanation != NULL ) {
            /*
             * Use status information from web service to generate an error message
             */
            if ( output_ls.retstatus->statusCode == SRM_USCOREFAILURE ) {
                /*
                 * Maybe user specified a wrong SURL
                 */
                len = strlen(output_ls.retstatus->explanation) + strlen(NO_DIR_ERROR);
                listing = (char* ) calloc( len + 1, sizeof(char) );
                if ( listing == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( listing, len, "%s%s", output_ls.retstatus->explanation, NO_DIR_ERROR );
            }
            else {
                len = strlen(output_ls.retstatus->explanation) + strlen(".\n");
                listing = (char* ) calloc( len + 1, sizeof(char) );
                if ( listing == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( listing, len, "%s.\n", output_ls.retstatus->explanation );
            }
            return status;
        }
        else if ( context.errbuf != NULL ) {
                len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
                listing = (char* ) calloc( len + 1, sizeof(char) );
                if ( listing == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( listing, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, output_ls.retstatus->explanation );
                return status;
        }
        else {
            /*
             * If there is no return status, something went wrong while setting up connection to web service
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            listing = (char *) calloc( len + 1, sizeof(char) );
            if ( listing == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( listing, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
            return status;
        }
    }
    else {
        if ( output_ls.statuses != NULL ) {

            listing = (char *) calloc( 5000, sizeof(char) );
            if ( listing == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            /*
             * If the user specified multiple SURLs, there will be also multiple items in the response
             * and we have to process each.
             */
            for ( i = 0; i < status; i++ ) {
                createResultString( &listing, &(output_ls.statuses[ i ]), 0 );
            }
	        return status;
        }
        else {
            /*
             * Pointer *statuses to struct srm2_mdfilestatus does not exist, no file information available
             */
            len = strlen(ERROR_INTRO) + strlen(STATUS_ERR_MSG);
            listing = (char *) calloc( len + 1, sizeof(char) );
            if ( listing == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( listing, len, "%s%s", ERROR_INTRO, STATUS_ERR_MSG );
            return status;
        }
    }
}

/**
 * \fn                  msiSrmLs
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surls - list of storage URLs
 * \param[in STR_MS_T]  subdirlevel - number of listed sub-directories
 * \param[out STR_MS_T] result - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmLs( msParam_t *endpoint, msParam_t *surls, msParam_t *subdirlevel, msParam_t *result, ruleExecInfo_t *rei ) {

    int stat;
    int nr_surls;

    long subdirlevel_long;

	char **surlsArray   = NULL;
	char **end_ptr      = NULL;

	struct srm_ls_input input_ls;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmLs: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointers
     */
    if ( endpoint == NULL
         || surls == NULL
         || subdirlevel == NULL
         || result == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmLs: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of input parameters
     */
    if ( endpoint->type == NULL
         || subdirlevel->type == NULL
         || surls->type == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmLs: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check input parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( subdirlevel->type, STR_MS_T ) != 0
         || strcmp( surls->type, STR_MS_T ) != 0 ) {

		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmLs: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
        return rei->status;
    }

    /*
     * Convert and check nr. of sub-directories, which will be processed
     */
    errno = 0;
    end_ptr = NULL;
    subdirlevel_long = strtol( (char* ) subdirlevel->inOutStruct, end_ptr, 10 );
    if ( (errno == ERANGE && (subdirlevel_long == LONG_MAX || subdirlevel_long == LONG_MIN))
         || (errno != 0 && subdirlevel_long == 0)
         || subdirlevel_long < 0
         || subdirlevel_long > INT_MAX
         || end_ptr != NULL ) {

   		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmLs: %s %s", INVALID_LEVEL_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Get separated SURLs from string
     */
    nr_surls = splitString( (char* ) surls->inOutStruct, &surlsArray, SPLITTER );
    if ( nr_surls <= 0 ) {

   		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmLs: %s %s", INVALID_SURL_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Set up srm_ls_input data structure
     */
	input_ls.count          = 0;                        // Count of elements to show
	input_ls.numlevels      = (int) subdirlevel_long;   // Number of folder levels for the listing
	input_ls.nbfiles        = nr_surls;                 // Number of files SURLs in the array
	input_ls.surls          = surlsArray;               // Array of SURLs
	input_ls.offset         = 0;                        // Offset of files to list

    stat = srm_ls_func( (char *) endpoint->inOutStruct, &input_ls );
    if ( stat == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmLs: %s", OUT_OF_MEM );
    }
    else if ( stat < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmLs: %s", listing );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( result, listing );
    }
    return rei->status;
}
