#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


/*
 * Set destination of debug output
 */
#define DEBUG_OUT                   stdout

#define SPLITTER                    ","
#define CORE_ERROR                  "\nPlease check specified storage URL.\n"
#define STAT_ERROR                  "Could not get status attributes of file "
#define NR_SURLS_FSIZES_UNEQUAL     "Number of specified storage URLs and file sizes is not equal.\n"
#define UNSUPPORTED_SRM_LONG64_TYPE "Type if SRM_LONG64 is not supported.\n"
#define RTOK_HEADER                 "Request Token: "


char* resultString = NULL;



int srm_preparetoput_func( char* endpoint, struct srm_preparetoput_input *input ) {

    int status;
    int len;
    int i;

	struct srm_context context;
    struct srm_preparetoput_output output;
    
    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_prepare_to_put() returns the number of files or -1 for error
     */
    status = srm_prepare_to_put( &context, input, &output );
    if ( status < 0 ) {
        /*
         * Error case - try multiple ways to get a meaningful error message
         */
        if ( output.retstatus != NULL && output.retstatus->explanation != NULL ) {
            /*
             * First: Use status information from web service to generate an error message
             */
            if ( output.retstatus->statusCode == SRM_USCOREFAILURE ) {
                /*
                 * Maybe user specified a wrong SURL
                 */
                len = strlen(output.retstatus->explanation) + strlen(CORE_ERROR);
                resultString = (char* ) calloc( len + 1, sizeof(char) );
                if ( resultString == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( resultString, len, "%s%s", output.retstatus->explanation, CORE_ERROR );
            }
            else {
                len = strlen(output.retstatus->explanation) + strlen(".\n");
                resultString = (char* ) calloc( len + 1, sizeof(char) );
                if ( resultString == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( resultString, len, "%s.\n", output.retstatus->explanation );
            }
            return status;
        }
        else if ( context.errbuf != NULL ) {
            /*
             * Second: Use information from SRM context error buffer
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
            return status;
        }
        else {
            /*
             * Finally: If there is no return status, something went wrong while setting up connection to web service
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
            return status;
        }
    }
    else {
        resultString =  calloc( 50000, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }
        // Print request token
        strncat( resultString, RTOK_HEADER, strlen(RTOK_HEADER) );
        if ( output.token != NULL ) {
            strncat( resultString, output.token, strlen(output.token) );
        }
        else {
            strncat( resultString, "NULL", strlen("NULL") );
        }
        strncat( resultString, "\n", strlen("\n") );
        for ( i = 0; i < status; i++ ) {
            printSrmPinFileStatus( i, &resultString, output.filestatuses[ i ] );
        }

        return status;
    }
}

/**
 * \fn                  msiSrmPrepareToPut
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  srcFiles - list of source files
 * \param[in STR_MS_T]  surls - list of storage URLs
 * \param[in STR_MS_T]  desc - space descriptor
 * \param[in STR_MS_T]  time - desired pin time
 * \param[in STR_MS_T]  protocols - list of data transfer protocols
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmPrepareToPut( msParam_t *endpoint, msParam_t *srcFiles, msParam_t *surls, msParam_t *desc, msParam_t *time, msParam_t *protocols, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

#ifdef SRM_DEBUG
    int debug_index;
#endif

	int status;
    int nr_surls;
    int nr_protocols;
    int len;
    int i;

    long time_long;

    SRM_LONG64* srcFilesLongArray   = NULL;
//  SRM_LONG64 temp;

    char* spaceTokenDescriptor;
    char** surlsArray               = NULL;
    char** protocolsArray           = NULL;
    char** srcFilesStringArray      = NULL;
    char** end_ptr                  = NULL;

    struct stat attributes;
    struct srm_preparetoput_input input;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmPrepareToPut: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL
         || srcFiles == NULL
         || surls == NULL
         || desc == NULL
         || time == NULL
         || protocols == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || srcFiles->type == NULL
         || surls->type == NULL
         || desc->type == NULL
         || time->type == NULL
         || protocols->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( srcFiles->type, STR_MS_T ) != 0
         || strcmp( surls->type, STR_MS_T ) != 0
         || strcmp( desc->type, STR_MS_T ) != 0
         || strcmp( time->type, STR_MS_T ) != 0
         || strcmp( protocols->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Split SURLs string into separated items
     */
    nr_surls = splitString( (char* ) surls->inOutStruct, &surlsArray, SPLITTER );
    if ( nr_surls <= 0 ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s %s", INVALID_SURL_SPEC, CHK_CMD );
		return rei->status;
    }

    /*
     * Handle user specified protocols for file transfer
     */
    nr_protocols = splitString( (char* ) protocols->inOutStruct, &protocolsArray, SPLITTER );
    if ( nr_protocols <= 0 ) {
        free( surlsArray );
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s %s", INVALID_PROTO_SPEC, CHK_CMD );
		return rei->status;
    }
    // Resize protocolsArray for one additional element (array should finish with an NULL element)
    protocolsArray = realloc( protocolsArray, (nr_protocols + 1) * sizeof(char*) );
    if ( protocolsArray == NULL ) {
        free( surlsArray );
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", OUT_OF_MEM );
        return rei->status;
    }
    // Set last element to NULL
    protocolsArray[ nr_protocols ] = NULL;

    /*
     * Get size of selected files
     */
    if ( splitString( (char* ) srcFiles->inOutStruct, &srcFilesStringArray, SPLITTER) != nr_surls ) {
        free( surlsArray );
        free( protocolsArray );
        // number of specified file sizes and storage URLs should be equal
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s %s", NR_SURLS_FSIZES_UNEQUAL, CHK_CMD );
        return rei->status;
    }
    else {
        srcFilesLongArray = calloc( nr_surls, sizeof(SRM_LONG64) );
        if ( srcFilesLongArray == NULL ) {

            free( surlsArray );
            free( protocolsArray );
            free( srcFilesStringArray );

            rei->status = EXEC_CMD_ERROR;
            rodsLogAndErrorMsg( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", OUT_OF_MEM );
            return rei->status;
        }

        for ( i = 0; i < nr_surls; i++ ) {
            if ( stat( srcFilesStringArray[ i ], &attributes ) == -1  ) {
                // An error occurred for getting status attributes of this file -> abort execution
                free( surlsArray );
                free( protocolsArray );
                free( srcFilesStringArray );
                free( srcFilesLongArray );

                len = strlen(ERROR_INTRO) + strlen(STAT_ERROR) + strlen("\n");
                // allocate also additional memory for one whitespace and two digits (and a final '\0')
                resultString = (char *) calloc( len + 4, sizeof(char) );
                if ( resultString == NULL ) {
                    rei->status = EXEC_CMD_ERROR;
                    rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", OUT_OF_MEM );
                    return rei->status;
                }
                snprintf( resultString, len + 3, "%s%s%d.\n", ERROR_INTRO, STAT_ERROR, i );

                rei->status = EXEC_CMD_ERROR;
                rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", resultString );
                return rei->status;
            }
            srcFilesLongArray[ i ] = attributes.st_size;
        }

        /*
         * Convert file sizes to SRM_LONG64 type
         */
/*        errno = 0;
        end_ptr = NULL;
        switch ( typeof_SRM_LONG64() ) {
            case SRM_LONG64_IS_LONG:
                for ( i = 0; i < nr_surls; i++ ) {
                    temp = strtol( srcFilesStringArray[ i ], end_ptr, 10 );*/
                    /*
                     * Check for various possible errors
                     */
/*                    if ( (errno == ERANGE && (temp == LONG_MAX || temp == LONG_MIN))
                         || (errno != 0 && temp == 0)
                         || temp < 0
                         || end_ptr != NULL ) {

                        free( surlsArray );
                        free( protocolsArray );
                        free( srcFilesStringArray );
                        free( srcFilesLongArray );
    
                        rei->status = EXEC_CMD_ERROR;
                        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", FILESIZE_CONVERSION_ERROR );
                        return rei->status;
                    }
                    srcFilesLongArray[ i ] = temp;
                }
                break;
            case SRM_LONG64_IS_LONG_LONG:
                for ( i = 0; i < nr_surls; i++ ) {
                    temp = strtoll( srcFilesStringArray[ i ], end_ptr, 10 );*/
                    /*
                     * Check for various possible errors
                     */
/*                    if ( (errno == ERANGE && (temp == LONG_MAX || temp == LONG_MIN))
                         || (errno != 0 && temp == 0)
                         || temp < 0
                         || end_ptr != NULL ) {

                        free( surlsArray );
                        free( protocolsArray );
                        free( srcFilesStringArray );
                        free( srcFilesLongArray );
    
                        rei->status = EXEC_CMD_ERROR;
                        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", FILESIZE_CONVERSION_ERROR );
                        return rei->status;
                    }
                    srcFilesLongArray[ i ] = temp;
                }
                break;
            default:
                // Error: unsupported type of SRM_LONG64
                free( surlsArray );
                free( protocolsArray );
                free( srcFilesStringArray );
                free( srcFilesLongArray );

                rei->status = EXEC_CMD_ERROR;
                rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", FILESIZE_CONVERSION_ERROR );
                return rei->status;
        }*/

    }

    /*
     * Convert desired pin time
     */
    errno = 0;
    end_ptr = NULL;
    time_long = strtol( (char* ) time->inOutStruct, end_ptr, 10 );
    if ( (errno == ERANGE && (time_long == LONG_MAX || time_long == LONG_MIN))
         || (errno != 0 && time_long == 0)
         || time_long < 0
         || time_long > INT_MAX
         || end_ptr != NULL ) {

        free( surlsArray );
        free( protocolsArray );
        free( srcFilesStringArray );
        free( srcFilesLongArray );

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", TIME_PARAM_INVALID_RANGE );
        return rei->status;
    }

    /*
     * Fill input data structure
     */
    input.filesizes         = srcFilesLongArray;
    input.nbfiles           = nr_surls;
    input.desiredpintime    = (int) time_long;
    input.surls             = surlsArray;
    input.protocols         = protocolsArray;
    spaceTokenDescriptor    = (char* ) desc->inOutStruct;
    if ( strcmp( spaceTokenDescriptor, "NULL" ) == 0 ) {
        input.spacetokendesc    = NULL;
    }
    else {
        input.spacetokendesc    = spaceTokenDescriptor;
    }
#ifdef SRM_DEBUG
    debug_index = 0;
    while (1) {
        if ( protocolsArray[ debug_index ] == NULL ) {
            fprintf( DEBUG_OUT, "protocolsArray[ %d ] = NULL\n", debug_index );
            break;
        }
        else {
            fprintf( DEBUG_OUT, "protocolsArray[ %d ] = %s\n", debug_index, input.protocols[ debug_index ] );
            debug_index++;
        }
    }
    fprintf( DEBUG_OUT, "Pin Time:\t\t\t%d\nSpace Token Descriptor:\t\t%s\nNr. SURLs:\t\t\t%d\n", input.desiredpintime, input.spacetokendesc, input.nbfiles );
    for ( debug_index = 0; debug_index < nr_surls; debug_index++ ) {
        fprintf( DEBUG_OUT, "\tSrc File %d: %s\n\t\tSize: %lld\n\n", debug_index, input.surls[debug_index], (long long int)input.filesizes[ debug_index] );
    }
#endif

    /*
     * Execute SRM command
     */
    status = srm_preparetoput_func( (char* ) endpoint->inOutStruct, &input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPrepareToPut: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    free( surlsArray );
    free( protocolsArray );
    free( srcFilesStringArray );
    free( srcFilesLongArray );
    return rei->status;
}
