#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


/*
 * Set destination of debug output
 */
#define DEBUG_OUT                   stdout

#define SPLITTER                    ","
#define INVALID_TOKEN_ERROR         "Invalid Request Token specified.\n"
#define DIGITS                      11


char* resultString = NULL;
char intToString[ DIGITS + 1 ];

int srm_put_done_func( char* endpoint, struct srm_putdone_input *input ) {

    int status;
    int len;
    int i;

	struct srm_context context;
    struct srmv2_filestatus *output;
    
    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_prepare_to_put() returns the number of files or -1 for error
     */
    status = srm_put_done( &context, input, &output );
    if ( status < 0 ) {
        if ( context.errbuf != NULL ) {
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
        }
        else {
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
        }
    }
    else {
        resultString =  calloc( 50000, sizeof(char) );
        for ( i = 0; i < input->nbfiles; i++ ) {
            printSrmFileStatus( i, &resultString, output[ i ] );
        }
    }
    return status;
}

/**
 * \fn                  msiSrmPutDone
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  requestToken - token generated by prior request
 * \param[in STR_MS_T]  surls - list of storage URLs
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 sometimes this operation fails
**/
int msiSrmPutDone( msParam_t *endpoint, msParam_t *requestToken, msParam_t *surls, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

	int status;
    int nr_surls;
#ifdef SRM_DEBUG
    int debug_index;
#endif

    long token;

    char** surlsArray   = NULL;
    char** end_ptr      = NULL;

    struct srm_putdone_input input;

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmPutDone: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL
         || requestToken == NULL
         || surls == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPutDone: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || requestToken->type == NULL
         || surls->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPutDone: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD  );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( requestToken->type, STR_MS_T ) != 0
         || strcmp( surls->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPutDone: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		return rei->status;
    }

    /*
     * Split strings into separated items
     */
    nr_surls = splitString( (char* ) surls->inOutStruct, &surlsArray, SPLITTER );
    if ( nr_surls <= 0 ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPutDone: %s %s", INVALID_SURL_SPEC, CHK_CMD );
		return rei->status;
    }

    /*
     * Check whether request token is valid or not
     */
    errno = 0;
    token = strtol( (char* ) requestToken->inOutStruct, end_ptr, 10 );
    if ( (errno == ERANGE && (token == LONG_MAX || token == LONG_MIN))
         || (errno != 0 && token == 0)
         || token < INT_MIN
         || token > INT_MAX
         || end_ptr != NULL ) {

        free( surlsArray );
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPutDone: %s %s", INVALID_TOKEN_ERROR, CHK_CMD );
		return rei->status;
    }

    /*
     * Fill input data structure
     */
    input.nbfiles   = nr_surls;
    input.surls     = surlsArray;
    input.reqtoken  = (char* ) requestToken->inOutStruct;

#ifdef SRM_DEBUG
    fprintf( DEBUG_OUT, "Nr. of Files:\t\t%d\n", input.nbfiles );
    for ( debug_index = 0; debug_index < input.nbfiles; debug_index++ ) {
        fprintf( DEBUG_OUT, "SURL[ %d ]:\t\t%s\n", debug_index, input.surls[ debug_index ] );
    }
    fprintf( DEBUG_OUT, "Request Token:\t\t%s\n", input.reqtoken );
#endif

    status = srm_put_done_func( (char* ) endpoint->inOutStruct, &input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPutDone: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmPutDone: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    free( surlsArray );
    return rei->status;
}
