#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define SPLITTER        ","
#define NO_DIR_ERROR    "\nMaybe the specified directory does not exist.\n"

char* resultString;         // string containing result information



int srm_rm_func( char *endpoint, struct srm_rm_input *input ) {

    int status;
    int len;
	int j;

	struct srm_context context;
	struct srm_rm_output output_rm;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_rm() returns the number of deleted files or -1 for error
     */
    status = srm_rm( &context, input, &output_rm );

	if ( status < 0 ) {
        /*
         *  Error case - we will try different ways to get a meaningful error message
         */
        if ( (output_rm.retstatus != NULL) && (output_rm.retstatus->explanation != NULL) ) {
            /*
             * First: Use status information from web service to generate an error message
             */
            if ( output_rm.retstatus->statusCode == SRM_USCOREFAILURE ) {
                /*
                 * Maybe user specified a wrong SURL
                 */
                len = strlen(output_rm.retstatus->explanation) + strlen(NO_DIR_ERROR);
                resultString = (char* ) calloc( len + 1, sizeof(char) );
                if ( resultString == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( resultString, len, "%s%s", output_rm.retstatus->explanation, NO_DIR_ERROR );
            }
            else {
                len = strlen(output_rm.retstatus->explanation) + strlen(".\n");
                resultString = (char* ) calloc( len + 1, sizeof(char) );
                if ( resultString == NULL ) {
                    return OUT_OF_MEMORY_EXCEPTION;
                }
                snprintf( resultString, len, "%s.\n", output_rm.retstatus->explanation );
            }
            return status;
        }
        else if ( context.errbuf != NULL ) {
            /*
             * Second: Use information from SRM context error buffer
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
            return status;
        }
        else {
            /*
             * Finally: If there is no return status, something went wrong while setting up connection to web service
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
            return status;
        }
    }
    else {
        resultString = calloc( 5000, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }

        /*
         * Print names of all deleted files
         */
        for ( j = 0; j < status; j++ ) {
            printSrmFileStatus( j, &resultString, output_rm.statuses[ j ] );
        }
    }
    return status;
}

/**
 * \fn                  msiSrmRm
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - list of storage URLs
 * \param[out STR_MS_T] result - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmRm( msParam_t *endpoint, msParam_t *surl, msParam_t *result, ruleExecInfo_t *rei ) {

    int stat;
    int nr_surls;

    char **surlsArray = NULL;

	struct srm_rm_input rm_input;
	
    if ( rei == NULL
         || rei->rsComm == NULL ) {

        rodsLog ( LOG_ERROR, "srmRm: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointers
     */
    if ( endpoint == NULL
         || surl == NULL
         || result == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRm: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || surl->type == NULL ) {

        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRm: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surl->type, STR_MS_T ) != 0 ) {

		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRm: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
        return rei->status;
    }

    /*
     * Get separated SURLs from string
     */
    nr_surls = splitString( (char* ) surl->inOutStruct, &surlsArray, SPLITTER );
    if ( nr_surls <= 0 ) {

   		rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRm: %s %s", INVALID_SURL_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Set up srm_rm_input data structure
     */
	rm_input.nbfiles        = nr_surls;         // Number of SURLs in the array
	rm_input.surls          = surlsArray;       // Array of SURLs

    stat = srm_rm_func( (char *) endpoint->inOutStruct, &rm_input );
    if ( stat == OUT_OF_MEMORY_EXCEPTION ) {
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRm: %s", OUT_OF_MEM );
    } else if ( stat < 0 ) {
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRm: %s", resultString );
    }
    else {
        rei->status = 1;
        fillStrInMsParam( result, resultString );
    }
    free( surlsArray );
    return rei->status;
}
