#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define INVALID_RECURSIVE_PARAMETER     "Only 0 or 1 is valid for recursive parameter."


char* resultString;

int srm_rmdir_func( char *endpoint, struct srm_rmdir_input *input ) {

    int status;
    int len;

    struct srm_context context;
    struct srm_rmdir_output output_rmdir;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );
    
    /*
     * Since revision 2496 of srm-ifce library srm_rmdir() 
     * returns 1 for success or -1 for error. In earlier
     * revisions the function returned 0 for success.
     */
	status = srm_rmdir( &context, input, &output_rmdir );
    if ( status == 1 ) {
        len = strlen(OK);
        resultString = (char *) calloc( len + 1, sizeof(char) );
        snprintf( resultString, len, "%s", OK );
    }
    else {
        /*
         * Error case
         */
        if ( output_rmdir.retstatus->explanation != NULL ) {
            /*
             * Maybe explanation of webservice response contains helpful information
             */
            len = strlen(output_rmdir.retstatus->explanation) + strlen(".\n");
            resultString = (char *) calloc( len + 1, sizeof(char) );
            snprintf( resultString, len, "%s.\n", output_rmdir.retstatus->explanation );
        }
        else if ( context.errbuf != NULL ) {
            /*
             * Use information from SRM context error buffer
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, INVALID_USER_PROXY_CERT, context.errbuf );
        }
        else {
            /*
             * If there is no return status, something went wrong while setting up connection to web service
             */
            len = strlen(ERROR_INTRO) + strlen(INVALID_USER_PROXY_CERT);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s", ERROR_INTRO, INVALID_USER_PROXY_CERT );
        }
    }
	return status;
}

/**
 * \fn                  msiSrmRmDir
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief
 * \param[in]           endpoint - is a msParam of type STR_MS_T
 * \param[in]           surl - is a msParam of type STR_MS_T
 * \param[in]           rec - is a msParam of type INT_MS_T
 * \param[out]          resultParameter - is a msParam of type STR_MS_T
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 no known bugs
**/
int msiSrmRmDir( msParam_t *endpoint, msParam_t *surl, msParam_t *recur, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int recursive;
    int status;

    struct srm_rmdir_input rmdir_input;


    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmRmDir: %s", NULL_REI_PARAMETER );
        return SYS_INTERNAL_NULL_INPUT_ERR;
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointers
     */
    if ( endpoint == NULL
         || surl == NULL
         || recur == NULL
         || resultParameter == NULL ) {
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRmDir: %s %s", NULL_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL
         || surl->type == NULL
         || recur->type == NULL ) {
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRmDir: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        return rei->status;
    }

    /*
     * Check type of input parameters
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surl->type, STR_MS_T ) != 0
         || strcmp( recur->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRmDir: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
        return rei->status;
    }

    /*
     * Only 0 or 1 is valid for recursive parameter
     */
    if ( strcmp( (char*) recur->inOutStruct, "0" ) == 0 ) {
        recursive = 0;
    }
    else if ( strcmp( (char*) recur->inOutStruct, "1" ) == 0 ) {
        recursive = 1;
    }
    else {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRmDir: %s %s", INVALID_RECURSIVE_PARAMETER, CHK_CMD );
        return rei->status;
    }

    /*
     * Set up srm_rmdir_input data structure
     */
    rmdir_input.recursive   = recursive;                    // Recursive (1) or not (0)
	rmdir_input.surl        = (char *) surl->inOutStruct;   // Directory SURL

    status = srm_rmdir_func( (char *) endpoint->inOutStruct, &rmdir_input );
    if ( status == 1 ) {
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    else if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRmDir: %s", OUT_OF_MEM );
    }
    else {
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "srmRmDir: %s", resultString );
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}
