#include <stdlib.h>

#include "srm.h"
#include "srmMS.h"

#include "srm_types.h"
#include "srm_ifce.h"
#include "srmv2Stub.h"


#define INVALID_PERM        -1
#define WRONG_PERMISSION    "Invalid permissions specified.\n"
#define FUNC_CALL_FAILURE   "Function srm_setpermission failed.\n"

/*
 * dCACHE does not support all input parameters, e.g.
 * -> user permissions are currently NOT SUPPORTED
 * -> group permissions has to use hyphen as group id
 */
#define DEFAULT_DUMMY_GROUP_ID  "-"


char* resultString = NULL;
struct srm_setpermission_input sp_input;


int srm_setpermission_func( char* endpoint, struct srm_setpermission_input *input ) {

    int status;

    size_t len;

	struct srm_context context;

    /*
     * Initialize context
     */
    initialize_srm_context( &context, endpoint );

    /*
     * srm_setpermission() returns 0 for success or -1 for error
     */
    status = srm_setpermission( &context, input );

    if ( status < 0 ) {
        /*
         *  Error case
         */
        if ( context.errbuf != NULL ) {
            len = strlen(ERROR_INTRO) + strlen(FUNC_CALL_FAILURE) + strlen("\n") + strlen(context.errbuf);
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n%s", ERROR_INTRO, FUNC_CALL_FAILURE, context.errbuf );
        }
        else {
            len = strlen(ERROR_INTRO) + strlen(FUNC_CALL_FAILURE) + strlen("\n");
            resultString = (char *) calloc( len + 1, sizeof(char) );
            if ( resultString == NULL ) {
                return OUT_OF_MEMORY_EXCEPTION;
            }
            snprintf( resultString, len, "%s%s\n", ERROR_INTRO, FUNC_CALL_FAILURE );
        }
    }
    else {
        len = strlen(OK) + strlen("\n");
        resultString = (char *) calloc( len + 1, sizeof(char) );
        if ( resultString == NULL ) {
            return OUT_OF_MEMORY_EXCEPTION;
        }
        snprintf( resultString, len, "%s\n", OK );
    }
    return status;
}

/*
 * Reads permissions string (specified on command line by the user) and returns corresponding TPermissionMode
 *
 * @param permission    permission as string specified by user
 * @return              corresponding TPermissionMode on success, INVALID_PERM on failure
 */
TPermissionMode checkPermission( char* permission ) {
    if ( strcmp( permission, "X" ) == 0 ) {
        return SRM_PERMISSION_X;
    } else if ( (strcmp( permission, "W" ) == 0) || (strcmp( permission, "w" ) == 0) ) {
        return SRM_PERMISSION_W;
    } else if ( (strcmp( permission, "WX" ) == 0) || (strcmp( permission, "wx" ) == 0) ) {
        return SRM_PERMISSION_WX;
    } else if ( (strcmp( permission, "R" ) == 0) || (strcmp( permission, "r" ) == 0) ) {
        return SRM_PERMISSION_R;
    } else if ( (strcmp( permission, "RX" ) == 0) || (strcmp( permission, "rx" ) == 0) ) {
        return SRM_PERMISSION_RX;
    } else if ( (strcmp( permission, "RW" ) == 0) || (strcmp( permission, "rw" ) == 0) ) {
        return SRM_PERMISSION_RW;
    } else if ( (strcmp( permission, "RWX" ) == 0) || (strcmp( permission, "rwx" ) == 0) ) {
        return SRM_PERMISSION_RWX;
    } else if ( (strcmp( permission, "NONE" ) == 0) || (strcmp( permission, "none" ) == 0) ) {
        return SRM_PERMISSION_NONE;
    } else {
        return INVALID_PERM;
    }
}


/*
 * Some tests to trap invalid parameters.
 *
 *	@param endpoint             msParam_t containing SRM manager URL
 *	@param surl                 msParam_t containing host URL
 *	@param permissionString     msParam_t containing permission (e.g., r, rw, wx, ...)
 *	@param resultParameter      msParam_t for result output
 *	@param rei                  iRODS ruleExecutionInfo structure
 */
void checkParametersV1( msParam_t *endpoint, msParam_t *surl, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmRm: %s", NULL_REI_PARAMETER );
        exit( SYS_INTERNAL_NULL_INPUT_ERR );
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL
         || surl == NULL
         || permissionString == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "checkParametersV1: %s %s", NULL_PARAMETER, CHK_CMD );
        exit( rei->status );
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL 
         || surl->type == NULL
         || permissionString->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "checkParametersV1: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        exit( rei->status );
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surl->type, STR_MS_T ) != 0
         || strcmp( permissionString->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "checkParametersV1: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		exit( rei->status );
    }
}

/*
 * Some tests to trap invalid parameters. This version handles an additional name parameter.
 *
 *	@param endpoint             msParam_t containing SRM manager URL
 *	@param surl                 msParam_t containing host URL
 *	@param name                 msParam_t containing group name
 *	@param permissionString     msParam_t containing permission (e.g., r, rw, wx, ...)
 *	@param resultParameter      msParam_t for result output
 *	@param rei                  iRODS ruleExecutionInfo structure
 */
void checkParametersV2( msParam_t *endpoint, msParam_t *surl, msParam_t *name, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    if ( rei == NULL || rei->rsComm == NULL ) {
        rodsLog ( LOG_ERROR, "srmRm: %s", NULL_REI_PARAMETER );
        exit( SYS_INTERNAL_NULL_INPUT_ERR );
    }
	setUserProxy( rei->rsComm );

    /*
     * Check parameter pointer
     */
    if ( endpoint == NULL 
         || surl == NULL 
         || name == NULL
         || permissionString == NULL
         || resultParameter == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "checkParametersV2: %s %s", NULL_PARAMETER, CHK_CMD );
        exit( rei->status );
    }

    /*
     * Check type pointer of parameters
     */
    if ( endpoint->type == NULL 
         || surl->type == NULL
         || name->type == NULL
         || permissionString->type == NULL ) {
        
        rei->status = SYS_INTERNAL_NULL_INPUT_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "checkParametersV2: %s %s", NO_PARAM_TYPE_SPEC, CHK_CMD );
        exit( rei->status );
    }

    /*
     * Check parameter types
     */
    if ( strcmp( endpoint->type, STR_MS_T ) != 0
         || strcmp( surl->type, STR_MS_T ) != 0
         || strcmp( name->type, STR_MS_T ) != 0
         || strcmp( permissionString->type, STR_MS_T ) != 0 ) {

        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "checkParametersV2: %s %s", INVALID_PARAM_TYPE, CHK_CMD );
		exit( rei->status );
    }
}



/*********************************************************************************************************/
/* iRODS Micro-Service Interface functions                                                               */
/*                                                                                                       */
/* srm_setpermission() supports three types to set new permissions: Add, Change, and Remove permissions. */
/* Furthermore separate permissions are managed for Owner, User, Group, and Other.                       */
/* To provide a user-friendly way of permission management, for each combination of type and entity a    */
/* specific iRODS Micro Service is implemented.                                                          */
/*********************************************************************************************************/

/***********************/
/*  OWNER PERMISSIONS  */
/***********************/


/**
 * \fn                  msiSrmAddOwnerPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Adds new Owner permissions
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmAddOwnerPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV1( endpoint, surl, permissionString, resultParameter, rei );

    /*
     * Check whether Owner permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddOwnerPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_ADD;
	sp_input.owner_permission           = permissionMode;
	sp_input.other_permission           = SRM_PERMISSION_NONE;
	sp_input.group_permissions_count    = 0;
	sp_input.group_permissions          = NULL;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddOwnerPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddOwnerPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}

/**
 * \fn                  msiSrmChangeOwnerPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Replaces existing Owner permissions with specified ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmChangeOwnerPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV1( endpoint, surl, permissionString, resultParameter, rei );

    /*
     * Check whether Owner permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeOwnerPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_CHANGE;
	sp_input.owner_permission           = permissionMode;
	sp_input.other_permission           = SRM_PERMISSION_NONE;
	sp_input.group_permissions_count    = 0;
	sp_input.group_permissions          = NULL;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeOwnerPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeOwnerPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}

/**
 * \fn                  msiSrmRemoveOwnerPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Removes specified Owner permissions from already existing ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmRemoveOwnerPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV1( endpoint, surl, permissionString, resultParameter, rei );

    /*
     * Check whether Owner permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveOwnerPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_REMOVE;
	sp_input.owner_permission           = permissionMode;
	sp_input.other_permission           = SRM_PERMISSION_NONE;
	sp_input.group_permissions_count    = 0;
	sp_input.group_permissions          = NULL;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveOwnerPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveOwnerPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}


/***********************/
/*  OTHER PERMISSIONS  */
/***********************/

/**
 * \fn                  msiSrmAddOtherPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Adds new Other permissions
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmAddOtherPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV1( endpoint, surl, permissionString, resultParameter, rei );

    /*
     * Check whether Other permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddOtherPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_ADD;
	sp_input.owner_permission           = SRM_PERMISSION_NONE;
	sp_input.other_permission           = permissionMode;
	sp_input.group_permissions_count    = 0;
	sp_input.group_permissions          = NULL;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddOtherPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddOtherPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}

/**
 * \fn                  msiSrmChangeOtherPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Replaces existing Other permissions with specified ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmChangeOtherPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV1( endpoint, surl, permissionString, resultParameter, rei );

    /*
     * Check whether Other permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeOtherPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_CHANGE;
	sp_input.owner_permission           = SRM_PERMISSION_NONE;
	sp_input.other_permission           = permissionMode;
	sp_input.group_permissions_count    = 0;
	sp_input.group_permissions          = NULL;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeOtherPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeOtherPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}

/**
 * \fn                  msiSrmRemoveOtherPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Removes specified Other permissions from already existing ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmRemoveOtherPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV1( endpoint, surl, permissionString, resultParameter, rei );

    /*
     * Check whether Other permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveOtherPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_REMOVE;
	sp_input.owner_permission           = SRM_PERMISSION_NONE;
	sp_input.other_permission           = permissionMode;
	sp_input.group_permissions_count    = 0;
	sp_input.group_permissions          = NULL;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveOtherPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveOtherPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}


/***********************/
/*  GROUP PERMISSIONS  */
/***********************/

/**
 * \fn                  msiSrmAddGroupPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Adds new Group permissions
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  groupName - group name
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmAddGroupPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *groupName, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    char* grp;

    TPermissionMode permissionMode;

    struct srm_permission permissionStruct;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV2( endpoint, surl, groupName, permissionString, resultParameter, rei );

    /*
     * Check whether Group permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddGroupPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_permission data structure
     */
    grp = (char* ) groupName->inOutStruct;
    if ( (strcmp( grp, "NULL" ) == 0) || (strcmp( grp, "null" ) == 0) ) {
        permissionStruct.name_id        = DEFAULT_DUMMY_GROUP_ID;
    }
    else {
        permissionStruct.name_id        = grp;
    }
    permissionStruct.mode               = permissionMode;

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_ADD;
	sp_input.owner_permission           = SRM_PERMISSION_NONE;
	sp_input.other_permission           = SRM_PERMISSION_NONE;
	sp_input.group_permissions_count    = 1;
	sp_input.group_permissions          = &permissionStruct;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddGroupPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddGroupPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}

/**
 * \fn                  msiSrmChangeGroupPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Replaces existing Group permissions with specified ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  groupName - group name
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmChangeGroupPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *groupName, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    char* grp;

    TPermissionMode permissionMode;

    struct srm_permission permissionStruct;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV2( endpoint, surl, groupName, permissionString, resultParameter, rei );


    /*
     * Check whether Group permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeGroupPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_permission data structure
     */
    grp = (char* ) groupName->inOutStruct;
    if ( (strcmp( grp, "NULL" ) == 0) || (strcmp( grp, "null" ) == 0) ) {
        permissionStruct.name_id        = DEFAULT_DUMMY_GROUP_ID;
    }
    else {
        permissionStruct.name_id        = grp;
    }
    permissionStruct.mode               = permissionMode;

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_CHANGE;
	sp_input.owner_permission           = SRM_PERMISSION_NONE;
	sp_input.other_permission           = SRM_PERMISSION_NONE;
	sp_input.group_permissions_count    = 1;
	sp_input.group_permissions          = &permissionStruct;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeGroupPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeGroupPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}

/**
 * \fn                  msiSrmRemoveGroupPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Removes specified Group permissions from already existing ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  groupName - group name
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmRemoveGroupPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *groupName, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    char* grp;

    TPermissionMode permissionMode;

    struct srm_permission permissionStruct;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV2( endpoint, surl, groupName, permissionString, resultParameter, rei );

    /*
     * Check whether Group permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveGroupPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_permission data structure
     */
    grp = (char* ) groupName->inOutStruct;
    if ( (strcmp( grp, "NULL" ) == 0) || (strcmp( grp, "null" ) == 0) ) {
        permissionStruct.name_id        = DEFAULT_DUMMY_GROUP_ID;
    }
    else {
        permissionStruct.name_id        = grp;
    }
    permissionStruct.mode               = permissionMode;

    /*
     * Set up srm_setpermission_input data structure
     */
  	sp_input.surl                       = (char *) surl->inOutStruct;
	sp_input.permission_type            = SRM_PERMISSION_REMOVE;
	sp_input.owner_permission           = SRM_PERMISSION_NONE;
	sp_input.other_permission           = SRM_PERMISSION_NONE;
	sp_input.group_permissions_count    = 1;
	sp_input.group_permissions          = &permissionStruct;
	sp_input.user_permissions_count     = 0;
	sp_input.user_permissions           = NULL;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveGroupPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveGroupPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}


/***********************/
/*  USER  PERMISSIONS  */
/***********************/

/*
 * Currently User Permissions Are Not Supportted By dCACHE
 */

/**
 * \fn                  msiSrmAddUserPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Adds specified User permissions to already existing ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  userName - user name
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmAddUserPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *userName, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    struct srm_permission permissionStruct;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV2( endpoint, surl, userName, permissionString, resultParameter, rei );

    /*
     * Check whether User permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddUserPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_permission data structure
     */
    permissionStruct.name_id            = (char* ) userName->inOutStruct;
    permissionStruct.mode               = permissionMode;

    /*
     * Set up srm_setpermission_input data structure
     */
    sp_input.surl                       = (char *) surl->inOutStruct;
    sp_input.permission_type            = SRM_PERMISSION_ADD;
    sp_input.owner_permission           = SRM_PERMISSION_NONE;
    sp_input.other_permission           = SRM_PERMISSION_NONE;
    sp_input.group_permissions_count    = 0;
    sp_input.group_permissions          = NULL;
    sp_input.user_permissions_count     = 1;
    sp_input.user_permissions           = &permissionStruct;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddUserPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmAddUserPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}

/**
 * \fn                  msiSrmChangeUserPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Replaces existing User permissions with specified ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  userName - user name
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmChangeUserPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *userName, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    struct srm_permission permissionStruct;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV2( endpoint, surl, userName, permissionString, resultParameter, rei );

    /*
     * Check whether User permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeUserPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_permission data structure
     */
    permissionStruct.name_id            = (char* ) userName->inOutStruct;
    permissionStruct.mode               = permissionMode;

    /*
     * Set up srm_setpermission_input data structure
     */
    sp_input.surl                       = (char *) surl->inOutStruct;
    sp_input.permission_type            = SRM_PERMISSION_CHANGE;
    sp_input.owner_permission           = SRM_PERMISSION_NONE;
    sp_input.other_permission           = SRM_PERMISSION_NONE;
    sp_input.group_permissions_count    = 0;
    sp_input.group_permissions          = NULL;
    sp_input.user_permissions_count     = 1;
    sp_input.user_permissions           = &permissionStruct;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeUserPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmChangeUserPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}


/**
 * \fn                  msiSrmRemoveUserPermission
 * \module              srm
 * \author              Ronny Tschueter
 * \date                October 2010
 * \brief               Removes specified User permissions from already existing ones
 * \param[in STR_MS_T]  endpoint - SRM manager URL
 * \param[in STR_MS_T]  surl - storage URLs
 * \param[in STR_MS_T]  userName - user name
 * \param[in STR_MS_T]  permissionString - permissions (e.g., r, rw, wx, ...)
 * \param[out STR_MS_T] resultParameter - output message as string
 * \param[in,out]       rei - The RuleExecInfo structure that is automatically
 *                      handled by the rule engine. The user does not include rei as a
 *                      parameter in the rule invocation.
 * \DolVarDependence    none
 * \DolVarModified      none
 * \iCatAtrDependence   none
 * \iCatAttrModified    none
 * \sideeffect          none
 * \return              integer
 * \retval              1 on success, otherwise: SYS_INTERNAL_NULL_INPUT_ERR, USER_PARAM_TYPE_ERR or EXEC_CMD_ERROR
 * \bug                 problems with dCache interoperability
**/
int msiSrmRemoveUserPermission( msParam_t *endpoint, msParam_t *surl, msParam_t *userName, msParam_t *permissionString, msParam_t *resultParameter, ruleExecInfo_t *rei ) {

    int status;

    TPermissionMode permissionMode;

    struct srm_permission permissionStruct;

    /*
     * Check whether all paramaters are valid
     */
    checkParametersV2( endpoint, surl, userName, permissionString, resultParameter, rei );

    /*
     * Check whether User permission is specified correctly or not
     */
    permissionMode = checkPermission( (char *) permissionString->inOutStruct );
    if ( permissionMode == INVALID_PERM ) {
        rei->status = USER_PARAM_TYPE_ERR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveUserPermission: %s", WRONG_PERMISSION );
        return rei->status;
    }

    /*
     * Set up srm_permission data structure
     */
    permissionStruct.name_id            = (char* ) userName->inOutStruct;
    permissionStruct.mode               = permissionMode;

    /*
     * Set up srm_setpermission_input data structure
     */
    sp_input.surl                       = (char *) surl->inOutStruct;
    sp_input.permission_type            = SRM_PERMISSION_REMOVE;
    sp_input.owner_permission           = SRM_PERMISSION_NONE;
    sp_input.other_permission           = SRM_PERMISSION_NONE;
    sp_input.group_permissions_count    = 0;
    sp_input.group_permissions          = NULL;
    sp_input.user_permissions_count     = 1;
    sp_input.user_permissions           = &permissionStruct;

    status = srm_setpermission_func( (char *) endpoint->inOutStruct, &sp_input );
    if ( status == OUT_OF_MEMORY_EXCEPTION ) {
        /* NO MORE MEMORY */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveUserPermission: %s", OUT_OF_MEM );
    }
    else if ( status < 0 ) {
        /* FAILURE */
        rei->status = EXEC_CMD_ERROR;
        rodsLogAndErrorMsg ( LOG_ERROR, &(rei->rsComm->rError), rei->status, "msiSrmRemoveUserPermission: %s", resultString );
    }
    else {
        /* SUCCESS */
        rei->status = 1;
        fillStrInMsParam( resultParameter, resultString );
    }
    return rei->status;
}
