/*
 *	@name		srm_tools.c
 *	@author		Ronny Tschueter, TU Dresden
 *	@license	BSD
 */

/***************************
 *
 *	Additional functions
 *
 **************************/
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include "srm.h"

#include "rodsLog.h"
#include "rsApiHandler.h"
#include "apiHeaderAll.h"
#include "rods.h"
#include "dataObjOpr.h"


#define NO_INFORMATION  "Information not available"
#define OUTP_HEADER     " File:          "
#define SURL_HEADER     " SURL:          "
#define TURL_HEADER     " TURL:          "
#define STAT_HEADER     " Status:        "
#define EXPL_HEADER     " Explanation:   "
#define PLFT_HEADER     " Pin Lifetime:  "

#define ERR_BUF_SIZE    5000

#define INT_DIGITS      11


/*
 *	Initializes context of SRM function calls.
 *
 *	@param context	        srm_context data structure
 *	@param endpoint	        URL of SRM manager
 *	@return			        0 on success, -1 on failure
 */
int initialize_srm_context( struct srm_context *context,char *srm_endpoint )
{
	context->version        = VERSION_2_2;
	context->srm_endpoint   = srm_endpoint;
	context->timeout        = 60;
	context->verbose        = 0;
	context->errbuf         = calloc( ERR_BUF_SIZE, sizeof(char) );
    if ( context->errbuf == NULL ) {
	    context->errbufsz       = 0;
        return -1;
    }
    else {
	    context->errbufsz       = ERR_BUF_SIZE;
        return 0;
    }   
}

/*
 *  Splits a string into separated tokens.
 *
 *	@param srcString        string that should be sperareted into tokens
 *  @param dstArray         array with all tokens
 *  @param splitter         token delimiter
 *	@return                 number of tokens in dstArray
 */
int splitString( char* srcString, char*** dstArray, char* splitter ) {
    
    int n = 0;
    int allocatedElements = 100;
    int allocIncrement = 100;
    char* ptr;
    char** tmp;

    /*
     * Initial memory allocation
     */
    *dstArray = calloc( allocatedElements, sizeof( char* ) );
    if ( *dstArray == NULL ) {
        return 0;
    }

    /*
     * Tokenize string
     */
    ptr = strtok( srcString, splitter );
    while ( ptr != NULL ) {
        (*dstArray)[ n ] = strndup( ptr, strlen(ptr) );
        ptr = strtok( NULL, splitter );
        n++;
        /*
         * Reallocate memory, if we will reach end of dstArray
         */
        if ( n == allocatedElements ) {
            tmp = realloc( *dstArray, (allocatedElements + allocIncrement)*sizeof(char*) );
            if ( tmp == NULL ) {
                /*
                 * Reallocation failed
                 */
                return n;
            }
            else {
                *dstArray = tmp;
                tmp = NULL;
                allocatedElements = allocatedElements + allocIncrement;
            }
        }
    }
    return n;
}

/*
 *  Determines type of SRM_LONG64.
 *  
 *  @return                 SRM_LONG64_IS_LONG if SRM_LONG64 has type long, or
 *                          SRM_LONG64_IS_LONG_LONG if SRM_LONG64 is of type long long
 */
int typeof_SRM_LONG64() {
    if ( strcmp(SRM_LONG64_FORMAT, "%ld") == 0 ) {
        return SRM_LONG64_IS_LONG;
    } else if ( strcmp(SRM_LONG64_FORMAT, "%lld") == 0 ) {
        return SRM_LONG64_IS_LONG_LONG;
    } else {
        return 0;
    }
}

/*
 *  Print all information from srmv2_filestatus data structure to given string.
 *
 *	@param index            integer, can be used to number output
 *  @param str              output string
 *	@param status           srmv2_filestatus data structure containing all information
 */
void printSrmFileStatus( int index, char** str, struct srmv2_filestatus status ) {

    char intToString[ INT_DIGITS + 1 ];

    strncat( *str, "\n", strlen("\n") );
    strncat( *str, OUTP_HEADER, strlen(OUTP_HEADER) );
    snprintf( intToString, INT_DIGITS, "%d", index );
    strncat( *str, intToString, strlen(intToString) );
    strncat( *str, "\n", strlen("\n") );

    // SURL
    strncat( *str, SURL_HEADER, strlen(SURL_HEADER) );
    if ( status.surl != NULL ) {
        strncat( *str, status.surl, strlen(status.surl) );
    }
    else {
        strncat( *str, NO_INFORMATION, strlen(NO_INFORMATION) );
    }
    strncat( *str, "\n", strlen("\n") );

    // TURL
    strncat( *str, TURL_HEADER, strlen(TURL_HEADER) );
    if ( status.turl != NULL ) {
        strncat( *str, status.turl, strlen(status.turl) );
    }
    else {
        strncat( *str, NO_INFORMATION, strlen(NO_INFORMATION) );
    }
    strncat( *str, "\n", strlen("\n") );

    // STATUS
    strncat( *str, STAT_HEADER, strlen(STAT_HEADER) );
    snprintf( intToString, INT_DIGITS, "%d\n", status.status );
    strncat( *str, intToString, strlen(intToString) );

    // EXPLANATION
    strncat( *str, EXPL_HEADER, strlen(EXPL_HEADER) );
    if ( status.explanation != NULL ) {
        strncat( *str, status.explanation, strlen(status.explanation) );
    }
    else {
        strncat( *str, NO_INFORMATION, strlen(NO_INFORMATION) );
    }
    strncat( *str, "\n", strlen("\n") );
}

/*
 *  Print all information from srmv2_pinfilestatus data structure to given string.
 *
 *	@param index            integer, can be used to number output
 *  @param str              output string
 *	@param status           srmv2_pinfilestatus data structure containing all information
 */
void printSrmPinFileStatus( int index, char** str, struct srmv2_pinfilestatus status ) {

    char intToString[ INT_DIGITS + 1 ];

    strncat( *str, "\n", strlen("\n") );
    strncat( *str, OUTP_HEADER, strlen(OUTP_HEADER) );
    snprintf( intToString, INT_DIGITS, "%d", index );
    strncat( *str, intToString, strlen(intToString) );
    strncat( *str, "\n", strlen("\n") );

    // SURL
    strncat( *str, SURL_HEADER, strlen(SURL_HEADER) );
    if ( status.surl != NULL ) {
        strncat( *str, status.surl, strlen(status.surl) );
    } else {
        strncat( *str, NO_INFORMATION, strlen(NO_INFORMATION) );
    }
    strncat( *str, "\n", strlen("\n") );

    // TURL
    strncat( *str, TURL_HEADER, strlen(TURL_HEADER) );
    if ( status.turl != NULL ) {
        strncat( *str, status.turl, strlen(status.turl) );
    } else {
        strncat( *str, NO_INFORMATION, strlen(NO_INFORMATION) );
    }
    strncat( *str, "\n", strlen("\n") );

    // STATUS
    strncat( *str, STAT_HEADER, strlen(STAT_HEADER) );
    snprintf( intToString, INT_DIGITS, "%d", status.status);
    strncat( *str, intToString, strlen(intToString) );
    strncat( *str, "\n", strlen("\n") );

    // EXPLANATION
    strncat( *str, EXPL_HEADER, strlen(EXPL_HEADER) );
    if ( status.explanation != NULL ) {
        strncat( *str, status.explanation, strlen(status.explanation) );
    } else {
        strncat( *str, NO_INFORMATION, strlen(NO_INFORMATION) );
    }
    strncat( *str, "\n", strlen("\n") );

    // PIN LIFE TIME
    strncat( *str, PLFT_HEADER, strlen(PLFT_HEADER) );
    snprintf( intToString, INT_DIGITS, "%d", status.pinlifetime);
    strncat( *str, intToString, strlen(intToString) );
    strncat( *str, "\n", strlen("\n") );
}

/*
 *	Prints an informational message to rodsLog, if SRM_DEBUG is defined.
 *
 *	@param formatStr    message to print
 */
void printInfo(char *formatStr, ...) {
#ifdef SRM_DEBUG
	int i;

	char msg[MSG_LEN];

	va_list ap;

	va_start(ap, formatStr);
	i = vsnprintf(msg, MSG_LEN - 1, formatStr, ap);
	va_end(ap);

	rodsLog(LOG_NOTICE, "SRM_MODULE: %s", msg);
#endif
}

/*
 *	Sets the user proxy certificate for use within microservices.
 *
 *	@param rsComm	        iRODS server communication handle
 *	@return			        0 on success, 1 on failure
 */
int setUserProxy( rsComm_t *rsComm ) {

	if ( NULL == getenv( "X509_USER_PROXY" ) ) {

        char* rodsHome = (char * ) &(rsComm->myEnv.rodsHome);

		if ( MAX_NAME_LEN < strlen( rodsHome ) + strlen( proxy_file ) ) {
			printInfo( "setUserProxy failed: buffer too small for filename" );
			return 1;
		}

		dataObjInfo_t *dataObjInfo = NULL;
		dataObjInp_t dataObjInp;
		bzero( &dataObjInp, sizeof( dataObjInp ) );

		strncpy( dataObjInp.objPath, ( rodsHome ), strlen( rodsHome ) );
		strncat( dataObjInp.objPath, proxy_file, strlen( proxy_file ) );

		if ( 0 == getDataObjInfo( rsComm, &dataObjInp, &dataObjInfo, NULL, 1 ) ) {

			setenv( "X509_USER_PROXY", dataObjInfo->filePath, 1 );
			printInfo( "X509_USER_PROXY: %s", getenv( "X509_USER_PROXY" ) );
		}
		else {
			printInfo( "rodsHome: %s", rsComm->myEnv.rodsHome );
			printInfo( "setUserProxy: getDataObjInfo failed for %s", dataObjInp.objPath );
			freeAllDataObjInfo( dataObjInfo );
			return 1;
		}
		freeAllDataObjInfo( dataObjInfo );
	}

	return 0;
}

